@@ -1,7 +1,31 @@
-/*-
- * copyright (c) 2004-2008 by Bruce Korb.  All rights reserved.
+#if defined(ENABLE_FMEMOPEN)
+#include <sys/ioctl.h>
+
+typedef enum {
+    FMEMC_INVALID       = 0,
+    FMEMC_GET_BUF_ADDR
+} fmemctl_t;
+
+typedef struct {
+    enum { FMEMC_GBUF_LEAVE_OWNERSHIP,
+           FMEMC_GBUF_TAKE_OWNERSHIP
+    }          own;
+    char *     buffer;
+    size_t     buf_size;
+    size_t     eof;
+} fmemc_get_buf_addr_t;
+
+#ifdef HURD
+#define _IOT__IOTBASE_fmemc_get_buf_addr_t sizeof(fmemc_get_buf_addr_t)
+#endif
+
+#define IOCTL_FMEMC_GET_BUF_ADDR \
+    _IOWR('m', FMEMC_GET_BUF_ADDR, fmemc_get_buf_addr_t)
+
+/**
+ * @file /old-home/bkorb/tools/mine/lib/fmemopen/fmemopen.c
  *
- * Time-stamp:      "2008-04-06 15:39:52 bkorb"
+ * Copyright (c) 2004-2011 by Bruce Korb.  All rights reserved.
  *
  * This code was inspired from software written by
  *   Hanno Mueller, kontakt@hanno.de
@@ -15,14 +39,11 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. Neither the name of the author nor the name of any other contributor
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -31,50 +52,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#if defined(ENABLE_FMEMOPEN)
-
-/*=--subblock=arg=arg_type,arg_name,arg_desc =*/
-/*=*
- * library:  fmem
- * header:   libfmem.h
- *
- * lib_description:
- *
- *  This library only functions in the presence of GNU or BSD's libc.
- *  It is distributed under the Berkeley Software Distribution License.
- *  This library can only function if there is a libc-supplied mechanism
- *  for fread/fwrite/fseek/fclose to call into this code.  GNU uses
- *  fopencookie and BSD supplies funopen.
-=*/
-/*
- * ag_fmemopen() - "my" version of a string stream
- * inspired by the glibc version, but completely rewritten and
- * extended by Bruce Korb - bkorb@gnu.org
- *
- * - See the man page.
- */
-
-#if defined(__linux) && ! defined(_GNU_SOURCE)
-#  define _GNU_SOURCE
-#endif
-
-#include <sys/types.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #if defined(HAVE_FOPENCOOKIE)
 #  if defined(HAVE_LIBIO_H)
 #    include <libio.h>
 #  endif
-   typedef _IO_off64_t  fmem_off_t;
-   typedef int          seek_pos_t;
+
+   typedef off64_t * seek_off_t;
+   typedef int       seek_ret_t;
 
 #elif defined(HAVE_FUNOPEN)
-   typedef size_t  fmem_off_t;
-   typedef fpos_t  seek_pos_t;
+   typedef fpos_t  seek_off_t;
+   typedef fpos_t  seek_ret_t;
 
 #  ifdef NEED_COOKIE_FUNCTION_TYPEDEFS
      typedef int     (cookie_read_function_t )(void *, char *, int);
@@ -82,10 +70,6 @@
      typedef fpos_t  (cookie_seek_function_t )(void *, fpos_t, int);
      typedef int     (cookie_close_function_t)(void *);
 #  endif /* NEED_COOKIE_FUNCTION_TYPEDEFS */
-
-#else
-#  error  OOPS
-   Choke Me.
 #endif
 
 #define PROP_TABLE \
@@ -98,10 +82,6 @@ _Prop_( truncate,   "start writing at start"  ) \
 _Prop_( allocated,  "we allocated the buffer" ) \
 _Prop_( fixed_size, "writes do not append"    )
 
-#ifndef NUL
-# define NUL '\0'
-#endif
-
 #define _Prop_(n,s)   BIT_ID_ ## n,
 typedef enum { PROP_TABLE BIT_CT } fmem_flags_e;
 #undef  _Prop_
@@ -114,7 +94,7 @@ typedef unsigned char buf_bytes_t;
 typedef struct fmem_cookie_s fmem_cookie_t;
 struct fmem_cookie_s {
     mode_bits_t    mode;
-    buf_bytes_t   *buffer;
+    buf_bytes_t *  buffer;
     size_t         buf_size;    /* Full size of buffer */
     size_t         next_ix;     /* Current position */
     size_t         eof;         /* End Of File */
@@ -123,41 +103,63 @@ struct fmem_cookie_s {
                                    by memory region. */
 };
 
+typedef struct {
+    FILE *          fp;
+    fmem_cookie_t * cookie;
+} cookie_fp_map_t;
+
+#ifndef AUTOGEN_BUILD
+/**
+ * invent our own boolean rather than try to work around all the ways
+ * different systems spell it.
+ */
+typedef enum { AG_FALSE = 0, AG_TRUE = 1 } ag_bool;
+#endif
+
+static cookie_fp_map_t const * map    = NULL;
+static unsigned int            map_ct = 0;
+static unsigned int            map_alloc_ct = 0;
+
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static int
-fmem_getmode( char const *pMode, mode_bits_t *pRes );
+fmem_getmode(char const * mode, mode_bits_t * pRes);
 
 static int
-fmem_extend( fmem_cookie_t *pFMC, size_t new_size );
+fmem_extend(fmem_cookie_t *pFMC, size_t new_size);
 
 static ssize_t
-fmem_read( void *cookie, void *pBuf, size_t sz );
+fmem_read(void *cookie, void *pBuf, size_t sz);
 
 static ssize_t
-fmem_write( void *cookie, const void *pBuf, size_t sz );
+fmem_write(void *cookie, const void *pBuf, size_t sz);
 
-static seek_pos_t
-fmem_seek( void *cookie, fmem_off_t *p_offset, int dir);
+static seek_ret_t
+fmem_seek(void * cookie, seek_off_t offset, int dir);
 
 static int
-fmem_close( void *cookie );
+fmem_close(void * cookie);
+
+static ag_bool
+fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len);
+
+static ag_bool
+fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len);
 /* = = = END-STATIC-FORWARD = = = */
 
-FILE * ag_fmemopen(void *buf, ssize_t len, char const *pMode);
-int fmem_ioctl( FILE* fp, int req, void* ptr );
-
 #ifdef TEST_FMEMOPEN
   static fmem_cookie_t* saved_cookie = NULL;
 #endif
 
+/**
+ * Convert a mode string into mode bits.
+ */
 static int
-fmem_getmode( char const *pMode, mode_bits_t *pRes )
+fmem_getmode(char const * mode, mode_bits_t * pRes)
 {
-    if (pMode == NULL)
+    if (mode == NULL)
         return 1;
 
-    switch (*pMode) {
+    switch (*mode) {
     case 'a': *pRes = FLAG_BIT(write) | FLAG_BIT(append);
               break;
     case 'w': *pRes = FLAG_BIT(write) | FLAG_BIT(truncate);
@@ -168,12 +170,12 @@ fmem_getmode( char const *pMode, mode_bits_t *pRes )
     }
 
     /*
-     *  If someone wants to supply a "wxxbxbbxbb+" pMode string, I don't care.
+     *  If someone wants to supply a "wxxbxbbxbb+" mode string, I don't care.
      */
     for (;;) {
-        switch (*++pMode) {
+        switch (*++mode) {
         case '+': *pRes |= FLAG_BIT(read) | FLAG_BIT(write);
-                  if (pMode[1] != NUL)
+                  if (mode[1] != NUL)
                       return EINVAL;
                   break;
         case NUL: break;
@@ -187,8 +189,11 @@ fmem_getmode( char const *pMode, mode_bits_t *pRes )
     return 0;
 }
 
+/**
+ * Extend the space associated with an fmem file.
+ */
 static int
-fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
+fmem_extend(fmem_cookie_t *pFMC, size_t new_size)
 {
     size_t ns = (new_size + (pFMC->pg_size - 1)) & (~(pFMC->pg_size - 1));
 
@@ -203,16 +208,16 @@ fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
          *  Previously, this was a user supplied buffer.  We now move to one
          *  of our own.  The user is responsible for the earlier memory.
          */
-        void* bf = malloc( ns );
+        void* bf = malloc(ns);
         if (bf == NULL)
             goto no_space;
 
-        memcpy( bf, pFMC->buffer, pFMC->buf_size );
+        memcpy(bf, pFMC->buffer, pFMC->buf_size);
         pFMC->buffer = bf;
         pFMC->mode  |= FLAG_BIT(allocated);
     }
     else {
-        void* bf = realloc( pFMC->buffer, ns );
+        void* bf = realloc(pFMC->buffer, ns);
         if (bf == NULL)
             goto no_space;
 
@@ -222,7 +227,7 @@ fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
     /*
      *  Unallocated file space is set to zeros.  Emulate that.
      */
-    memset( pFMC->buffer + pFMC->buf_size, 0, ns - pFMC->buf_size );
+    memset(pFMC->buffer + pFMC->buf_size, 0, ns - pFMC->buf_size);
     pFMC->buf_size = ns;
     return 0;
 
@@ -231,8 +236,11 @@ fmem_extend( fmem_cookie_t *pFMC, size_t new_size )
     return -1;
 }
 
+/**
+ * Handle file system callback to read data from our string.
+ */
 static ssize_t
-fmem_read( void *cookie, void *pBuf, size_t sz )
+fmem_read(void *cookie, void *pBuf, size_t sz)
 {
     fmem_cookie_t *pFMC = cookie;
 
@@ -249,9 +257,11 @@ fmem_read( void *cookie, void *pBuf, size_t sz )
     return sz;
 }
 
-
+/**
+ * Handle file system callback to write data to our string
+ */
 static ssize_t
-fmem_write( void *cookie, const void *pBuf, size_t sz )
+fmem_write(void *cookie, const void *pBuf, size_t sz)
 {
     fmem_cookie_t *pFMC = cookie;
     int add_nul_char;
@@ -277,7 +287,7 @@ fmem_write( void *cookie, const void *pBuf, size_t sz )
     {
         size_t next_pos = pFMC->next_ix + sz + add_nul_char;
         if (next_pos > pFMC->buf_size) {
-            if (fmem_extend( pFMC, next_pos ) != 0) {
+            if (fmem_extend(pFMC, next_pos) != 0) {
                 /*
                  *  We could not extend the memory.  Try to write some data.
                  *  Fail if we are either at the end or not writing data.
@@ -295,7 +305,7 @@ fmem_write( void *cookie, const void *pBuf, size_t sz )
         }
     }
 
-    memcpy( pFMC->buffer + pFMC->next_ix, pBuf, sz);
+    memcpy(pFMC->buffer + pFMC->next_ix, pBuf, sz);
 
     pFMC->next_ix += sz;
 
@@ -316,107 +326,180 @@ fmem_write( void *cookie, const void *pBuf, size_t sz )
     return sz;
 }
 
-
-static seek_pos_t
-fmem_seek( void *cookie, fmem_off_t *p_offset, int dir)
+/**
+ * Handle file system callback to set a new current position
+ */
+static seek_ret_t
+fmem_seek(void * cookie, seek_off_t offset, int dir)
 {
     size_t new_pos;
     fmem_cookie_t *pFMC = cookie;
 
+#ifdef HAVE_FOPENCOOKIE
+    /*
+     *  GNU interface:  offset passed and returned by address.
+     */
     switch (dir) {
-    case SEEK_SET: new_pos = *p_offset;  break;
-    case SEEK_CUR: new_pos = pFMC->next_ix  + *p_offset;  break;
-    case SEEK_END: new_pos = pFMC->eof - *p_offset;  break;
+    case SEEK_SET: new_pos = *offset;  break;
+    case SEEK_CUR: new_pos = pFMC->next_ix  + *offset;  break;
+    case SEEK_END: new_pos = pFMC->eof - *offset;  break;
 
     default:
         goto seek_oops;
     }
+#else
+    /*
+     *  BSD interface:  offset passed by value, returned as retval.
+     */
+    switch (dir) {
+    case SEEK_SET: new_pos = offset;  break;
+    case SEEK_CUR: new_pos = pFMC->next_ix  + offset;  break;
+    case SEEK_END: new_pos = pFMC->eof - offset;  break;
+
+    default:
+        goto seek_oops;
+    }
+#endif
 
     if ((signed)new_pos < 0)
         goto seek_oops;
 
     if (new_pos > pFMC->buf_size) {
-        if (fmem_extend( pFMC, new_pos ))
+        if (fmem_extend(pFMC, new_pos))
             return -1; /* errno is set */
     }
 
     pFMC->next_ix = new_pos;
+
+#ifdef HAVE_FOPENCOOKIE
+    *offset = (off64_t)new_pos;
+    return 0;
+#else
     return new_pos;
+#endif
 
  seek_oops:
     errno = EINVAL;
     return -1;
 }
 
-
+/**
+ * Free up the memory associated with an fmem file.
+ * If the user is managing the space, then the allocated bit is set.
+ */
 static int
-fmem_close( void *cookie )
+fmem_close(void * cookie)
 {
-    fmem_cookie_t *pFMC = cookie;
+    fmem_cookie_t   * pFMC = cookie;
+    cookie_fp_map_t * pmap = (void *)map;
+    unsigned int      mct  = map_ct;
+
+    while (mct-- != 0) {
+        if (pmap->cookie == cookie) {
+            *pmap = map[--map_ct];
+            break;
+        }
+        pmap++;
+    }
+
+    if (mct > map_ct)
+        errno = EINVAL;
 
     if (pFMC->mode & FLAG_BIT(allocated))
-        free( pFMC->buffer );
-    free( pFMC );
+        free(pFMC->buffer);
+    free(pFMC);
 
     return 0;
 }
 
-/*=export_func fmem_ioctl
- *
- *  what:  get information about a string stream
- *
- *  arg: + FILE* + fptr  + the string stream
- *  arg: + int   + req   + the requested data
- *  arg: + void* + ptr   + ptr to result area
- *
- *  ret-type:  int
- *  ret-desc:  zero on success
- *
- *  err: non-zero is returned and @code{errno} is set to @code{EINVAL}.
- *
- *  doc:
- *
- *  This routine surreptitiously slips in a special request.
- *  The commands supported are:
- *
- *  @table @code
- *  @item FMEM_IOCTL_BUF_ADDR
- *
- *    Retrieve the address of the buffer.  Future output to the stream might
- *    cause this buffer to be freed and the contents copied to another buffer.
- *    You must ensure that either you have saved the buffer (see
- *    @code{FMEM_IOCTL_SAVE_BUF} below), or do not do any more I/O to it while
- *    you are using this address.
- *
- *    "ptr" must point to a @code{char*} pointer.
- *
- *  @item FMEM_IOCTL_SAVE_BUF
- *
- *    Do not deallocate the buffer on close.  You would likely want to use
- *    this after writing all the output data and just before closing.
- *    Otherwise, the buffer might get relocated.  Once you have specified
- *    this, the current buffer becomes the client program's resposibility to
- *    @code{free()}.  If more I/O operations are performed, a new buffer
- *    @i{may} get allocated.  @code{fmem_close} will free that new buffer
- *    and the user will remain responsible for @code{free()}-ing this buffer.
- *
- *    "ptr" must point to a @code{char*} pointer.
- *
- *  @end table
- *
- *  The third argument is never optional and must be a pointer to where data
- *  are to be retrieved or stored.  It may be NULL if there are no data to
- *  transfer, but both of these functions currently return the address of the
- *  buffer.  This is implemented as a wrapper around @code{fseek(3C)}, so
- *  the "req" argument must not conflict with @code{SEEK_SET}, @code{SEEK_CUR}
- *  or @code{SEEK_END}.
-=*/
-int
-fmem_ioctl( FILE* fp, int req, void* ptr )
+/**
+ * Configure the user supplied buffer.
+ */
+static ag_bool
+fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len)
 {
-    if (fseek( fp, (long)ptr, req ) < 0)
-        return -1;
-    return 0;
+    /*
+     *  User allocated buffer.  User responsible for disposal.
+     */
+    if (len == 0) {
+        free(pFMC);
+        errno = EINVAL;
+        return AG_FALSE;
+    }
+
+    pFMC->buffer = (buf_bytes_t*)buf;
+
+    /*  Figure out where our "next byte" and EOF are.
+     *  Truncated files start at the beginning.
+     */
+    if (pFMC->mode & FLAG_BIT(truncate)) {
+        /*
+         *  "write" mode
+         */
+        pFMC->eof = \
+            pFMC->next_ix = 0;
+    }
+
+    else if (pFMC->mode & FLAG_BIT(binary)) {
+        pFMC->eof = len;
+        pFMC->next_ix    = (pFMC->mode & FLAG_BIT(append)) ? len : 0;
+
+    } else {
+        /*
+         * append or read text mode -- find the end of the buffer
+         * (the first NUL character)
+         */
+        buf_bytes_t *p = (buf_bytes_t*)buf;
+
+        pFMC->eof = 0;
+        while ((*p != NUL) && (++(pFMC->eof) < len))  p++;
+        pFMC->next_ix =
+            (pFMC->mode & FLAG_BIT(append)) ? pFMC->eof : 0;
+    }
+
+    /*
+     *  text mode - NUL terminate buffer, if it fits.
+     */
+    if (  ((pFMC->mode & FLAG_BIT(binary)) == 0)
+       && (pFMC->next_ix < len)) {
+        pFMC->buffer[pFMC->next_ix] = NUL;
+    }
+
+    pFMC->buf_size = len;
+    return AG_TRUE;
+}
+
+/**
+ * Allocate an initial buffer for fmem.
+ */
+static ag_bool
+fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len)
+{
+    /*
+     *  We must allocate the buffer.  If "len" is zero, set it to page size.
+     */
+    pFMC->mode |= FLAG_BIT(allocated);
+    if (len == 0)
+        len = pFMC->pg_size;
+
+    /*
+     *  Unallocated file space is set to NULs.  Emulate that.
+     */
+    pFMC->buffer = calloc((size_t)1, (size_t)len);
+    if (pFMC->buffer == NULL) {
+        errno = ENOMEM;
+        free(pFMC);
+        return AG_FALSE;
+    }
+
+    /*
+     *  We've allocated the buffer.  The end of file and next entry
+     *  are both zero.
+     */
+    pFMC->next_ix  = 0;
+    pFMC->eof      = 0;
+    pFMC->buf_size = len;
+    return AG_TRUE;
 }
 
 /*=export_func ag_fmemopen
@@ -437,14 +520,13 @@ fmem_ioctl( FILE* fp, int req, void* ptr )
  *  This function requires underlying @var{libc} functionality:
  *  either @code{fopencookie(3GNU)} or @code{funopen(3BSD)}.
  *
- *  If @var{buf} is @code{NULL}, then a buffer is allocated.
- *  The initial allocation is @var{len} bytes.  If @var{len} is
- *  less than zero, then the buffer will not be reallocated if more
- *  space is needed.  Any allocated memory is @code{free()}-ed
- *  when @code{fclose(3C)} is called.
+ *  If @var{buf} is @code{NULL}, then a buffer is allocated.  The initial
+ *  allocation is @var{len} bytes.  If @var{len} is less than zero, then the
+ *  buffer will be reallocated as more space is needed.  Any allocated
+ *  memory is @code{free()}-ed when @code{fclose(3C)} is called.
  *
  *  If @code{buf} is not @code{NULL}, then @code{len} must not be zero.
- *  It may still be less than zero to indicate that the buffer is not to
+ *  It may still be less than zero to indicate that the buffer may
  *  be reallocated.
  *
  *  The mode string is interpreted as follows.  If the first character of
@@ -475,10 +557,10 @@ fmem_ioctl( FILE* fp, int req, void* ptr )
  *  The buffer is marked as updatable and both reading and writing is enabled.
  *
  *  @item b
- *  The I/O is marked as "binary" and a trailing NUL will not be inserted into
- *  the buffer.  Without this mode flag, one will be inserted after the
+ *  The I/O is marked as "binary" and a trailing NUL will not be inserted
+ *  into the buffer.  Without this mode flag, one will be inserted after the
  *  @code{EOF}, if it fits.  It will fit if the buffer is extensible (the
- *  provided @var{len} was not negative).  This mode flag has no effect if
+ *  provided @var{len} was negative).  This mode flag has no effect if
  *  the buffer is opened in read-only mode.
  *
  *  @item x
@@ -489,14 +571,14 @@ fmem_ioctl( FILE* fp, int req, void* ptr )
  *  Any other letters following the inital 'a', 'w' or 'r' will cause an error.
 =*/
 FILE *
-ag_fmemopen(void *buf, ssize_t len, char const *pMode)
+ag_fmemopen(void * buf, ssize_t len, char const * mode)
 {
     fmem_cookie_t *pFMC;
 
     {
-        mode_bits_t mode;
+        mode_bits_t bits;
 
-        if (fmem_getmode(pMode, &mode) != 0) {
+        if (fmem_getmode(mode, &bits) != 0) {
             return NULL;
         }
 
@@ -506,7 +588,7 @@ ag_fmemopen(void *buf, ssize_t len, char const *pMode)
             return NULL;
         }
 
-        pFMC->mode = mode;
+        pFMC->mode = bits;
     }
 
     /*
@@ -514,133 +596,156 @@ ag_fmemopen(void *buf, ssize_t len, char const *pMode)
      *  a negative size implies fixed size buffer and a NULL
      *  buffer pointer means we must allocate (and free) it.
      */
-    if (len < 0) {
-        len = -len;
-        pFMC->mode |= FLAG_BIT(fixed_size);
-    }
-
-    else
+    if (len <= 0) {
         /*
          *  We only need page size if we might extend an allocation.
          */
+        len = -len;
         pFMC->pg_size = getpagesize();
-
-    if (buf != NULL) {
-        /*
-         *  User allocated buffer.  User responsible for disposal.
-         */
-        if (len == 0) {
-            free( pFMC );
-            errno = EINVAL;
-            return NULL;
-        }
-
-        /*  Figure out where our "next byte" and EOF are.
-         *  Truncated files start at the beginning.
-         */
-        if (pFMC->mode & FLAG_BIT(truncate)) {
-            /*
-             *  "write" mode
-             */
-            pFMC->eof = \
-            pFMC->next_ix = 0;
-        }
-
-        else if (pFMC->mode & FLAG_BIT(binary)) {
-            pFMC->eof = len;
-            pFMC->next_ix    = (pFMC->mode & FLAG_BIT(append)) ? len : 0;
-
-        } else {
-            /*
-             * append or read text mode -- find the end of the buffer
-             * (the first NUL character)
-             */
-            buf_bytes_t *p = pFMC->buffer = (buf_bytes_t*)buf;
-            pFMC->eof = 0;
-            while ((*p != NUL) && (++(pFMC->eof) < len))  p++;
-            pFMC->next_ix =
-                (pFMC->mode & FLAG_BIT(append)) ? pFMC->eof : 0;
-        }
-
-        /*
-         *  text mode - NUL terminate buffer, if it fits.
-         */
-        if (  ((pFMC->mode & FLAG_BIT(binary)) == 0)
-           && (pFMC->next_ix < len)) {
-            pFMC->buffer[pFMC->next_ix] = NUL;
-        }
     }
 
-    else if ((pFMC->mode & (FLAG_BIT(append) | FLAG_BIT(truncate))) == 0) {
+    else {
+        pFMC->mode |= FLAG_BIT(fixed_size);
+    }
+
+    if (buf != NULL) {
+        if (! fmem_config_user_buf(pFMC, buf, len))
+            return NULL;
+
+    } else if ((pFMC->mode & (FLAG_BIT(append) | FLAG_BIT(truncate))) == 0) {
         /*
          *  Not appending and not truncating.  We must be reading.
          *  We also have no user supplied buffer.  Nonsense.
          */
         errno = EINVAL;
-        free( pFMC );
+        free(pFMC);
         return NULL;
     }
 
-    else {
-        /*
-         *  We must allocate the buffer.  If "len" is zero, set it to page size.
-         */
-        pFMC->mode |= FLAG_BIT(allocated);
-        if (len == 0)
-            len = pFMC->pg_size;
-
-        /*
-         *  Unallocated file space is set to NULs.  Emulate that.
-         */
-        pFMC->buffer = calloc((size_t)1, (size_t)len);
-        if (pFMC->buffer == NULL) {
-            errno = ENOMEM;
-            free( pFMC );
-            return NULL;
-        }
-
-        /*
-         *  We've allocated the buffer.  The end of file and next entry
-         *  are both zero.
-         */
-        pFMC->next_ix = 0;
-        pFMC->eof = 0;
-    }
-
-    pFMC->buf_size   = len;
+    else if (! fmem_alloc_buf(pFMC, len))
+        return NULL;
 
 #ifdef TEST_FMEMOPEN
     saved_cookie = pFMC;
 #endif
 
     {
+        FILE * res;
+
         cookie_read_function_t* pRd = (pFMC->mode & FLAG_BIT(read))
             ?  (cookie_read_function_t*)fmem_read  : NULL;
         cookie_write_function_t* pWr = (pFMC->mode & FLAG_BIT(write))
             ? (cookie_write_function_t*)fmem_write : NULL;
-#if defined(HAVE_FOPENCOOKIE)
+
+#ifdef HAVE_FOPENCOOKIE
         cookie_io_functions_t iof;
         iof.read  = pRd;
         iof.write = pWr;
-        iof.seek  = (cookie_seek_function_t* )fmem_seek;
-        iof.close = (cookie_close_function_t*)fmem_close;
+        iof.seek  = fmem_seek;
+        iof.close = fmem_close;
 
-        return fopencookie( pFMC, pMode, iof );
-#elif defined(HAVE_FUNOPEN)
-        return funopen( pFMC, pRd, pWr,
-                        (cookie_seek_function_t* )fmem_seek,
-                        (cookie_close_function_t*)fmem_close );
+        res = fopencookie(pFMC, mode, iof);
 #else
-#       include "We have neither fopencookie(3GNU) nor funopen(3BSD)"
+        res = funopen(pFMC, pRd, pWr, fmem_seek, fmem_close);
 #endif
+        if (res == NULL)
+            return res;
+
+        if (++map_ct >= map_alloc_ct) {
+            void * p = (map_alloc_ct > 0)
+                ? realloc((void *)map, (map_alloc_ct += 4) * sizeof(*map))
+                : malloc((map_alloc_ct = 4) * sizeof(*map));
+
+            if (p == NULL) {
+                fclose(res);
+                errno = ENOMEM; /* "fclose" set it to "EINVAL". */
+                return NULL;
+            }
+
+            map = p;
+        }
+
+        {
+            cookie_fp_map_t * p = (void *)(map + map_ct - 1);
+            p->fp = res;
+            p->cookie = pFMC;
+        }
+
+        return res;
     }
 }
 
-#endif /* ENABLE_FMEMOPEN */
+/*=export_func ag_fmemioctl
+ *
+ *  what:  perform an ioctl on a FILE* descriptor
+ *
+ *  arg: + FILE* + fp      + file pointer  +
+ *  arg: + int   + req     + ioctl command +
+ *  arg: + ...   + varargs + arguments for command +
+ *
+ *  ret-type:  int
+ *  ret-desc:  zero on success, otherwise error in errno
+ *
+ *  err:  errno is set to @code{EINVAL} or @code{ENOSPC}.
+ *
+ *  doc:
+ *
+ *  The file pointer passed in must have been returned by ag_fmemopen.
+=*/
+int
+ag_fmemioctl(FILE * fp, int req, ...)
+{
+    fmem_cookie_t * cookie;
+    fmemc_get_buf_addr_t * gba;
+
+    if (req != IOCTL_FMEMC_GET_BUF_ADDR) {
+        /*
+         *  It is not any of the IOCTL commands we know about.
+         */
+        errno = EINVAL;
+        return -1;
+    }
+
+    {
+        cookie_fp_map_t const * pmap = map;
+        unsigned int mct  = map_ct;
+
+        for (;;) {
+            if (mct-- == 0) {
+                /*
+                 *  fmemopen didn't create this FILE*, so it is invalid.
+                 */
+                errno = EINVAL;
+                return -1;
+            }
+            if (pmap->fp == fp)
+                break;
+            pmap++;
+        }
+
+        cookie = pmap->cookie;
+    }
+
+    {
+        va_list ap;
+        va_start(ap, req);
+        gba = va_arg(ap, fmemc_get_buf_addr_t *);
+        va_end(ap);
+    }
+
+    gba->buffer   = (char *)(cookie->buffer);
+    gba->buf_size = cookie->buf_size;
+    gba->eof      = cookie->eof;
+    if (gba->own != FMEMC_GBUF_LEAVE_OWNERSHIP)
+        cookie->mode &= ~FLAG_BIT(allocated);
+    return 0;
+}
+
 /*
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
  * indent-tabs-mode: nil
  * End:
- * end of agen5/fmemopen.c */
+ * end of fmemopen.c */
+#endif /* ENABLE_FMEMOPEN */
