@@ -1,10 +1,10 @@
 
 /**
- * \file funcDef.c
+ * @file funcDef.c
  *
  *  This module implements the DEFINE text function.
  *
- *  Time-stamp:        "2011-01-27 12:32:10 bkorb"
+ *  Time-stamp:        "2011-06-05 15:31:33 bkorb"
  *
  *  This file is part of AutoGen.
  *  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
@@ -51,7 +51,7 @@ static void
 fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac);
 
 static void
-prep_invoke_args(tMacro* pMac);
+prep_invoke_args(tMacro * pMac);
 
 static void
 build_defs(int defCt, tDefList* pList);
@@ -208,7 +208,7 @@ fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac)
         default:
             AG_ABEND_IN(pT, pMac, "name not followed by '='");
 
-        case ' ': case '\t': case '\n': case '\f':
+        case ' ': case TAB: case NL: case '\f':
             *(pzScan++) = NUL;
             while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
 
@@ -338,10 +338,10 @@ parseMacroArgs(tTemplate* pT, tMacro* pMac)
 }
 
 static void
-prep_invoke_args(tMacro* pMac)
+prep_invoke_args(tMacro * pMac)
 {
-    char*  pzText;
-    tTemplate* pT = pCurTemplate;
+    char * pzText;
+    tTemplate * pT = pCurTemplate;
 
     if (pMac->ozText == 0)
         AG_ABEND_IN(pT, pMac, "The INVOKE macro requires a name");
@@ -395,7 +395,7 @@ prep_invoke_args(tMacro* pMac)
  *      helpful, but not crucial.  Please contact the author if you think you
  *      might actually want to use this.
 =*/
-tMacro*
+tMacro *
 mFunc_Debug(tTemplate* pT, tMacro* pMac)
 {
     static int dummy = 0;
@@ -438,7 +438,7 @@ mFunc_Debug(tTemplate* pT, tMacro* pMac)
     }
     if (IS_GRAPHIC_CHAR(dummy))
         fprintf(pfTrace, " (%c)", dummy);
-    putc('\n', pfTrace);
+    putc(NL, pfTrace);
     return pMac+1;
 }
 
@@ -466,7 +466,7 @@ build_defs(int defCt, tDefList* pList)
         switch (*(pList->pzExpr)) {
         case ';':
         {
-            char* pz = strchr(pList->pzExpr, '\n');
+            char* pz = strchr(pList->pzExpr, NL);
             if (pz != NULL) {
                 while (IS_WHITESPACE_CHAR(*++pz))  ;
                 pList->pzExpr = pz;
@@ -682,8 +682,8 @@ mUnload_Define(tMacro* pMac)
  *  the first time the macro is processed and evaluated again every
  *  time the macro is evaluated.
 =*/
-tMacro*
-mFunc_Invoke(tTemplate* pT, tMacro* pMac)
+tMacro *
+mFunc_Invoke(tTemplate * pT, tMacro * pMac)
 {
     char* pzText;
     SCM   macName;
@@ -735,14 +735,13 @@ mFunc_Invoke(tTemplate* pT, tMacro* pMac)
     return mFunc_Define(pT, pMac);
 }
 
-
 /* Load Debug
  *
  *  what:   Loads the debug function so you can set breakpoints
  *          at load time, too :-)
  */
-tMacro*
-mLoad_Debug(tTemplate* pT, tMacro* pMac, char const ** ppzScan)
+tMacro *
+mLoad_Debug(tTemplate * pT, tMacro * pMac, char const ** ppzScan)
 {
     if (OPT_VALUE_TRACE >= TRACE_DEBUG_MESSAGE)
         return mLoad_Unknown(pT, pMac, ppzScan);
@@ -750,13 +749,13 @@ mLoad_Debug(tTemplate* pT, tMacro* pMac, char const ** ppzScan)
 }
 
 
-tMacro*
-mLoad_Define(tTemplate* pT, tMacro* pMac, char const ** ppzScan)
+tMacro *
+mLoad_Define(tTemplate * pT, tMacro * pMac, char const ** ppzScan)
 {
     static char const zNameNeeded[] = "DEFINE requires a name";
 
-    char*        pzCopy;             /* next text dest   */
-    tTemplate*   pNewT;
+    char *       pzCopy;             /* next text dest   */
+    tTemplate *  pNewT;
 
     /*
      *  Save the global macro loading mode
