@@ -1,7 +1,7 @@
 
 /**
  *  \file load.c
- *  Time-stamp:      "2010-12-06 14:03:01 bkorb"
+ *  Time-stamp:      "2011-05-24 18:05:28 bkorb"
  *
  *  This file contains the routines that deal with processing text strings
  *  for options, either from a NUL-terminated string passed in or from an
@@ -9,7 +9,7 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -28,8 +28,6 @@
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
-tOptionLoadMode option_load_mode = OPTION_LOAD_UNCOOKED;
-
 /* = = = START-STATIC-FORWARD = = = */
 static ag_bool
 insertProgramPath(char * pzBuf, int bufSize, char const * pzName,
@@ -61,8 +59,8 @@ assembleArgValue(char* pzTxt, tOptionLoadMode mode);
  *
  * doc:
  *
- *  This routine will copy the @code{pzName} input name into the @code{pzBuf}
- *  output buffer, carefully not exceeding @code{bufSize} bytes.  If the
+ *  This routine will copy the @code{pzName} input name into the
+ *  @code{pzBuf} output buffer, not exceeding @code{bufSize} bytes.  If the
  *  first character of the input name is a @code{'$'} character, then there
  *  is special handling:
  *  @*
@@ -98,17 +96,9 @@ ag_bool
 optionMakePath(char * pzBuf, int bufSize, char const * pzName,
                char const * pzProgPath)
 {
-    size_t  name_len = strlen(pzName);
+    size_t name_len = strlen(pzName);
 
-#   ifndef PKGDATADIR
-#     define PKGDATADIR ""
-#   endif
-
-    static char const pkgdatadir[] = PKGDATADIR;
-
-    ag_bool res = AG_TRUE;
-
-    if (bufSize <= name_len)
+    if ((bufSize <= name_len) || (name_len == 0))
         return AG_FALSE;
 
     /*
@@ -137,46 +127,52 @@ optionMakePath(char * pzBuf, int bufSize, char const * pzName,
         return AG_FALSE;
 
     case '$':
-        res = insertProgramPath(pzBuf, bufSize, pzName, pzProgPath);
+        if (! insertProgramPath(pzBuf, bufSize, pzName, pzProgPath))
+            return AG_FALSE;
         break;
 
     case '@':
-        if (pkgdatadir[0] == NUL)
+        if (program_pkgdatadir[0] == NUL)
             return AG_FALSE;
 
-        if (name_len + sizeof (pkgdatadir) > bufSize)
+        if (snprintf(pzBuf, bufSize, "%s%s", program_pkgdatadir, pzName + 2)
+            >= bufSize)
             return AG_FALSE;
-
-        strcpy(pzBuf, pkgdatadir);
-        strcpy(pzBuf + sizeof(pkgdatadir) - 1, pzName + 2);
         break;
 
     default:
-        res = insertEnvVal(pzBuf, bufSize, pzName, pzProgPath);
+        if (! insertEnvVal(pzBuf, bufSize, pzName, pzProgPath))
+            return AG_FALSE;
     }
 
-    if (! res)
-        return AG_FALSE;
-
 #if defined(HAVE_CANONICALIZE_FILE_NAME)
     {
-        char* pz = canonicalize_file_name(pzBuf);
+        char * pz = canonicalize_file_name(pzBuf);
         if (pz == NULL)
             return AG_FALSE;
-        if (strlen(pz) < bufSize)
-            strcpy(pzBuf, pz);
+
+        name_len = strlen(pz);
+        if (name_len >= bufSize) {
+            free(pz);
+            return AG_FALSE;
+        }
+
+        memcpy(pzBuf, pz, name_len + 1);
         free(pz);
     }
 
 #elif defined(HAVE_REALPATH)
     {
-        char z[ PATH_MAX+1 ];
+        char z[PATH_MAX+1];
 
         if (realpath(pzBuf, z) == NULL)
             return AG_FALSE;
 
-        if (strlen(z) < bufSize)
-            strcpy(pzBuf, z);
+        name_len = strlen(z);
+        if (name_len >= bufSize)
+            return AG_FALSE;
+
+        memcpy(pzBuf, z, name_len + 1);
     }
 #endif
 
@@ -376,7 +372,7 @@ loadOptionLine(
     {
         char* pzArg = assembleArgValue(pzLine, load_mode);
 
-        if (! SUCCESSFUL(longOptionFind(pOpts, pzLine, pOS)))
+        if (! SUCCESSFUL(opt_find_long(pOpts, pzLine, pOS)))
             return;
         if (pOS->flags & OPTST_NO_INIT)
             return;
