@@ -1,4 +1,3 @@
-#line 1 "./filament.in"
 /*  -*- Mode: C -*-  */
 
 /* filament.h --- a bit like a string but different =)O|
@@ -41,7 +40,7 @@ extern "C"
 #endif
 #endif				/* __cplusplus */
 
-#define FILAMENT_BUFSIZ       512
+#define FILAMENT_BUFSIZ       (512 - sizeof(char *) - (2 * sizeof(size_t)))
 
 /**
  * Filament:
@@ -59,7 +58,7 @@ struct filament
 };
 
 /**
- * filnew:  constructor 
+ * filnew: constructor
  * @init: address of the first byte to copy into the new object.
  * @len:  the number of bytes to copy into the new object.
  *
@@ -71,11 +70,11 @@ struct filament
  * Return value:
  * A newly created Filament object is returned.
  **/
-extern Filament * filnew (const char *const init, size_t len);
-
+extern Filament *
+filnew (const char *const init, size_t len);
 
 /**
- * filinit:   
+ * filinit:
  * @fil: The Filament object to initialise.
  * @init: address of the first byte to copy into the new object.
  * @len:  the number of bytes to copy into the new object.
@@ -87,11 +86,11 @@ extern Filament * filnew (const char *const init, size_t len);
  * Return value:
  * The initialised Filament object is returned.
  **/
-extern Filament * filinit (Filament *fil, const char *const init, size_t len);
-
+extern Filament *
+filinit (Filament *fil, const char *const init, size_t len);
 
 /**
- * fildelete:  destructor 
+ * fildelete: destructor
  * @fil: The Filament object for recycling.
  *
  * The memory being used by @fil is recycled.
@@ -102,11 +101,11 @@ extern Filament * filinit (Filament *fil, const char *const init, size_t len);
  * as a normal C string.  The entire Filament contents are copied into
  * this string including any embedded nulls.
  **/
-extern char * fildelete (Filament *fil);
-
+extern char *
+fildelete (Filament *fil);
 
 /**
- * _fil_extend:   
+ * _fil_extend:
  * @fil: The Filament object which may need more string space.
  * @len: The length of the data to be stored in @fil.
  * @copy: whether to copy data from the static buffer on reallocation.
@@ -115,10 +114,10 @@ extern char * fildelete (Filament *fil);
  * considering the space left in @fil and @len, the length required
  * for the prospective contents.
  */
-extern void _fil_extend (Filament *fil, size_t len, snv_bool_t copy);
+extern void
+_fil_extend (Filament *fil, size_t len, snv_bool_t copy);
 
-
-#line 61 "./filament.in"
+#line 61 "filament.in"
 
 /* Save the overhead of a function call in the great majority of cases. */
 #define fil_maybe_extend(fil, len, copy)  \
@@ -157,6 +156,27 @@ fillen (Filament *fil)
   return fil->length;
 }
 
+/**
+ * filelt:
+ * @fil: The Filament being queried.
+ * @n: A zero based index into @fil.
+ *
+ * This function looks for the @n'th element of @fil.
+ *
+ * Return value:
+ * If @n is an index inside the Filament @fil, then the character stored
+ * at that index cast to an int is returned, otherwise @n is outside
+ * this range and -1 is returned.
+ **/
+SNV_INLINE int
+filelt (Filament *fil, ssize_t n)
+{
+  if ((n >= 0) && (n < fil->length))
+    return (int) fil->value[n];
+  else
+    return -1;
+}
+
 /**
  * filncat:
  * @fil: The destination Filament of the concatenation.
